<?php
namespace Teknisa\EST\Modules\EstoqueCego\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;

class Est91000ProgramacaoDoInventarioCegoService {
    
    protected $Est91000ProgramacaoDoInventarioCegoFactory;
    protected $dataSourceManager;
    
    public function __construct($Est91000factory, $environment,$dataSourceManager) {
        $this->Est91000ProgramacaoDoInventarioCegoFactory = $Est91000factory;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
    }
    
    public function formatMutipleFilialCode (&$conditions){
        
        foreach($conditions as &$condition){
            if($condition['columnName'] = 'CDFILIAL'){
                $newCondition = join("_",$condition['value']);
                $condition['value'] = "_" . $newCondition . "_";;
                break;
            }
        }
    }
    
    public function insertLote ($row){
        $params = array(
            ":CDFILIAL" => $row['CDFILIAL'],
            ":NRLOTEESTQ" => $row['NRLOTEESTQ'],
            ":NRSUBLOTE" => $row['NRSUBLOTE'],
            ":CDPRODUTO"   => $row['CDPRODUTO']
        );
        $this->Est91000ProgramacaoDoInventarioCegoFactory->insertLote($params);
    }
    
    public function checkInventoryActualDate ($row){
        $params = array(
            ":CDFILIAL" => $row['CDFILIAL'],
            ":DTCONTAGEM" => $row['DTCONTAGEM']
        );
        
        return $this->Est91000ProgramacaoDoInventarioCegoFactory->checkInventoryActualDate($params);
    }
    
    public function deleteAllInventory($row) {
        $params = array(
            ":CDFILIAL" => $row['CDFILIAL'],
            ":DTCONTAGEM" => $row['DTCONTAGEM']
        );
        
        $this->Est91000ProgramacaoDoInventarioCegoFactory->deleteAllInventory($params);
    }
    
    public function checkHolidays($row) {
        $params = array(
            ":CDFILIAL" => $row['CDFILIAL'],
            ":DTCONTAGEM" => $row['DTCONTAGEM']
        );
        
        $isHoliday = $this->Est91000ProgramacaoDoInventarioCegoFactory->checkHolidays($params);
        return !empty($isHoliday) ? "S" : "N";
    }
    
    public function getProductsInfo($row, $filial) {
        $dtInicial = $row['INTERVALO']['start'];
        $dtFinal = $row['INTERVALO']['end'];
        $row['CDPRODINI'] = empty($row['CDPRODINI']) ? "00000000000000000000" : str_pad($row['CDPRODINI'], 20,"0");
        $row['CDPRODFIN'] = empty($row['CDPRODFIN']) ? "zzzzzzzzzzzzzzzzzzzz" : str_pad($row['CDPRODFIN'], 20,"0");
        
            if(!empty($row['CDGRUPPROD'])){ // apenas CDGRUPPROD
                
                $params = [":CDGRUPPROD" => $row['CDGRUPPROD']];
                $result = $this->Est91000ProgramacaoDoInventarioCegoFactory->getCDFILISUBGR($params);
                
                $params = [
                    "CDFILISUBGR"=>$result['CDFILISUBGR'],
                    "CDGRUPPROD"=>$row['CDGRUPPROD'], 
                    "CDPRODINI"=>$row['CDPRODINI'],
                    "CDPRODFIN"=>$row['CDPRODFIN'],
                    "DTINICIAL"=>$dtInicial,
                    "DTFINAL"=>$dtFinal,
                    "CDFILIAL"=>$filial
                ];
                $complementaryInfo = $this->Est91000ProgramacaoDoInventarioCegoFactory->getProductsInfo($params,"byGroupOnly");
                
            }else if((empty($row['CDGRUPPROD'])) && (!empty($row['NMPRODINI']) && !empty($row['NMPRODFIN']))){ // apenas prodini e prodfim
                
                $params = [
                    "CDPRODINI"=>$row['CDPRODINI'],
                    "CDPRODFIN"=>$row['CDPRODFIN'],
                    "DTINICIAL"=>$dtInicial,
                    "DTFINAL"=>$dtFinal,
                    "CDFILIAL"=>$filial
                ];
                $complementaryInfo = $this->Est91000ProgramacaoDoInventarioCegoFactory->getProductsInfo($params,"byProductsOnly");
            
            }else if(true){ // grupo/parametro != de zero -> marcado
                
                // $complementaryInfo = $this->Est91000ProgramacaoDoInventarioCegoFactory->getProductsInfo($params,"byGroupWithParamChecked");
            }else{ // apenas prodini e prodfim + parametro != de zero -> marcado
                
                // $complementaryInfo = $this->Est91000ProgramacaoDoInventarioCegoFactory->getProductsInfo($params,"byProductsWithParamChecked");
            }
        return $complementaryInfo;
        
    }
    
    public function getInventoriesToSave ($row,$filial,$complementaryInfo){
        $InventoriesArray = array();
        foreach ($complementaryInfo as $info) {
            if(!empty($InventoriesArray)){
                foreach($InventoriesArray as $inventory){
                    $checkBool = $info['CDALMOXARIFE'] == $inventory['CDALMOXARIFE'] && 
                    $info['CDLOCALESTOQ'] == $inventory['CDLOCALESTOQ'] ? false : true;
                }
            }else{ $checkBool = true; }
            
            if($checkBool){
        		$info["CDFILIAL"] = $filial;
                $info["CDOPERADOR"] = $row["CDOPERADOR"];
                $info["DTCONTAGEM"] = $row["DTCONTAGEM"];
                $info["DTGERACAOINVEST"] = date("d/m/y");
                $info["IDSITESTCEGO"] = "PR";
                $info["IDSITUACEITEINV"] = "P";
                $info["QTINVENTESTCEGO"] = null;
        		$info["__is_new"] = true;
        		$info["changed"] = true;
        		array_push($InventoriesArray, $info);
            }
        }
        return $InventoriesArray;
    }
    
    public function getProductsToSave ($row,$filial,$complementaryInfo){
        $productsArray = array();
        
        foreach ($complementaryInfo as $product) {
    		$product["CDFILIAL"] = $filial;
            $product["DTCONTAGEM"] = $row["DTCONTAGEM"];
            $product["NRLOTEESTQ"] = $product["NRLOTEESTQ"];
            $product["NRSUBLOTE"] = $product["NRSUBLOTE"];
            $product["CDPRODUTO"] = $product["CDPRODMOVI"];
            $product["QTINVENTESTCEGO"] = "0";
            $product["IDGERITINVCEGPR"] = "M";
    		$product["__is_new"] = true;
    		$product["changed"] = true;
    		array_push($productsArray, $product);
        }
        
        return $productsArray;
    }
    
    
}
