<?php
namespace Folha\Logic\ESocial\Eventos\Data;
use Folha\Repositories\GpeEsocialRegistro;

/**
 * Classe provedora do datasource utilizado para a geracao do evento S-1350 - Aquisição de Produção.
 *
 * @package Folha\Logic\ESocial\Eventos\Data
 * @author felipesantos
 **/
class S1350
{

    /**
     * Retorna os dados que serao utilizado para a geracao do(s) arquivo(s) do evento S-1350, Aquisição de Produção.
     *
     * @param $dsordem                 Define as possíveis formas de filtro utilizadas pela tela de envio de eventos  
     *                                 do e-Social. Pode ser ALFABETICA, MATRICULA, DATA_ADMISSAO e ESTRUTURA. Para este
     *                                 evento, em especial, somente o filtro por ESTRUTURA poderá será relevante.
     * @param $limiteInferior          Limite inferior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $limiteSuperior          Limite superior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $inicioPeriodo           Inicio do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true". 
     * @param $fimPeriodo              Final do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true".
     * @param $utilizaIntervaloDeDatas Verifica se o dataset deve ser filtrado pelos campos de intervalo da tela de  
     *                                 envio de eventos e-Social, ou se devem enviados os arquivos alterados depois do 
     *                                 ultimo envio.
     * @param $idRetificacao           Identificador de Retificação dos arquivos ou não.
     * @param $nrRecibo                Número do Recibo do primeiro envio em caso de Retificação.
     * @param $idApuracao              Identificador da apuração Mensal ou Anual.
     * @param $nrorg                   Nrorg.
     * @param $versaoHRM               Versao do sistema HRM utilizado para a geracao dos relatorios.
     *
     * @return array
     * @author felipesantos
     **/
    public static function getDataSource($dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo, 
                                         $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $idRetificacao, $nrRecibo, $idApuracao)
    {
        $S1350 = GpeEsocialRegistro::dataSetEsocial_S1350(
            $dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo, 
            $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $idRetificacao, $nrRecibo, $idApuracao
        );

        // Loop para gerar os estabelecimentos adquirentes referentes ao empregador acima
        foreach ($S1350 as &$row) {
            $estabs = GpeEsocialRegistro::s13xx_EstabPrestTomad('E', $row['NRESTRUTURAM'], $inicioPeriodo, $fimPeriodo, $nrorg);

            // Loop para gerar os tipos de aquisição referentes ao estabelecimento adquirente acima
            $contEstab = 1;
            $estabAdquir = null;
            foreach($estabs as &$estab) {
                $aquisicoes = GpeEsocialRegistro::s1350_tipoAquisicao($estab['CDFILIRECE'], $inicioPeriodo, $fimPeriodo);

                // Loop para gerar os produtores referentes ao tipo de aquisição acima
                $contAquis = 1;
                $tpAquisicao = null;
                foreach($aquisicoes as &$aquisicao) {
                    $prods = GpeEsocialRegistro::s13xx_PrestadTomador('S', $estab['CDFILIRECE'], $inicioPeriodo, $fimPeriodo, $aquisicao['IDTPIJURFORN']);

                    // Loop para gerar as notas referentes ao produtor acima
                    $contProd = 1;
                    $produtor = null;
                    foreach($prods as &$prod) {
                        $notas = GpeEsocialRegistro::s13xx_NfServPrestTomad('E', $estab['CDFILIRECE'], $prod['CDFORNECED'], $inicioPeriodo, $fimPeriodo);

                        // Loop para gerar os itens referentes à nota acima
                        $contNF = 1;
                        $notasfiscais = null;
                        foreach ($notas as &$notafiscal) {
                            $notasfiscais[$contNF] =
                                array("_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_NOTASFISCAIS" => '1',
                                    "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_NOTASFISCAIS_SERIE" => $notafiscal['SERIE'],
                                    "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_NOTASFISCAIS_NUMDOCTO" => $notafiscal['NUMDOCTO'],
                                    "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_NOTASFISCAIS_DTEMISSAONF" => $notafiscal['DTEMISSAONF'],
                                    "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_NOTASFISCAIS_VLRBRUTO" => $notafiscal['NFVLRBRUTO'],
                                    "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_NOTASFISCAIS_VLRCONTRIBDESCPR" => $notafiscal['NFVLRCONTRIBDESCPR'],
                                    "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_NOTASFISCAIS_VLRGILRATDESCPR" => $notafiscal['NFVLRGILRATDESCPR'],
                                    "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_NOTASFISCAIS_VLRSENARDESCPR" => $notafiscal['NFVLRSENARDESCPR']);
                            $contNF++;
                        }

                        $produtor[$contProd] =
                            array("_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR" => '1',
                                "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_TPINSCRICAO" => $prod['PRODTPINSCRICAO'],
                                "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_NRINSCRICAO" => $prod['PRODNRINSCRICAO'],
                                "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_VLRBRUTO" => $prod['VLRBRUTO'],
                                "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_VLRCONTRIBDESCPR" => $prod['VLRCONTRIBDESCPR'],
                                "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_VLRGILRATDESCPR" => $prod['VLRGILRATDESCPR'],
                                "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_VLRSENARDESCPR" => $prod['VLRSENARDESCPR'],
                                "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR_NOTASFISCAIS" => $notasfiscais);
                        $contProd++;

                    }

                    $tpAquisicao[$contAquis] =
                        array("_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO" => '1',
                            "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_INDAQUISICAO" => $aquisicao['INDAQUISICAO'],
                            "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_VLRTOTALAQUISICAO" => $aquisicao['VLRTOTALAQUISICAO'],
                            "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO_IDEPRODUTOR" => $produtor);
                    $contAquis++;
                }

                $estabAdquir[$contEstab] =
                    array("_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE" => '1',
                        "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TPINSCRICAO" => $estab['ESTABTPINSCRICAO'],
                        "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_NRINSCRICAO" => $estab['ESTABNRINSCRICAO'],
                        "_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE_TIPOAQUISICAO" => $tpAquisicao);
                $contEstab++;
            }

            $row['_EVTFPAQUISPRODUCAO_INFOAQUISPRODUCAO_IDEESTABADQUIRENTE'][1] = $estabAdquir;
        }

        return $S1350;
    }

}