<?php

namespace Geral\Logic\OfxParser;

/**
 * An OFX parser library.
 * 
 * @author Guillaume Bailleul <contact@guillaume-bailleul.fr>
 * @author Adaptado por Matheus Silva <matheus.silva@teknisa.com>
 */
class OfxParser {

    private $OFXContent;
    private $originalString;
    private $moviments;
    private $conf;
    private $bankingData;
    private $fileData;


    private function fixTag() {
        $explode = explode("\n", $this->OFXContent);
        $array = array();
        for($i=0; $i<count($explode); $i++){
            $line = $explode[$i];
            $pos = strpos(trim($line), '>');
            $after = substr(trim($line), ($pos + 1));
            $sub = strlen(trim(substr($line, strpos($line, '>'))));
            $openTag = trim(substr($line, (strpos($line, '<') + 1)));
            $tag = strtoupper(substr($openTag, 0, -($sub)));
            if((strpos($line, "<$tag>") == 0 || strpos($line, "<$tag>") != false) && !strpos($line, "</$tag>") && ($after === '0' || $after) && ($tag != 'STMTTRN')){
                $array[] = $line . "</$tag>";
            }
            else{
                $array[] = $line;
            }
        }
        $this->OFXContent = implode("\n", $array);
    }

    public function loadFromFile($OFXFile) {
        if (file_exists($OFXFile)){
            $this->loadFromString(file_get_contents($OFXFile));
            return true;
        }
        else{
            return false;
        }
    }

    public function loadFromString($OFXContent) {
        $this->originalString = $OFXContent;
        $this->OFXContent = $OFXContent;

        $this->OFXContent = explode('<OFX>', $this->OFXContent);
        $this->conf = $this->OFXContent[0];
        $this->OFXContent = '<OFX>' . $this->OFXContent[1];
        $this->fixTag();
        
        $xmlContent = "<?xml version='1.0'?> " . $this->OFXContent;

        $this->ofx = simplexml_load_string(utf8_decode($xmlContent));

        $conf = explode('/r', trim($this->conf));
        $this->conf = Array();

        for ($i = 0, $j = sizeof($conf); $i < $j; $i++) {
            $conf[$i] = explode(':', $conf[$i], 2);
            $this->conf[$conf[$i][0]] = $conf[$i][1];
        }
        $this->moviments = Array();

        foreach ($this->ofx->BANKMSGSRSV1->STMTTRNRS->STMTRS->BANKTRANLIST->STMTTRN as $mov) {
            $this->moviments[] = (array) $mov;
        }
        $this->bankingData= array();

        foreach($this->ofx->BANKMSGSRSV1->STMTTRNRS->STMTRS->BANKACCTFROM as $bank){
            $this->bankingData[] = (array) $bank;
        }

        $this->fileData = array(
            'DTSTART' => (string) $this->ofx->BANKMSGSRSV1->STMTTRNRS->STMTRS->BANKTRANLIST->DTSTART,
            'DTEND' => (string)$this->ofx->BANKMSGSRSV1->STMTTRNRS->STMTRS->BANKTRANLIST->DTEND,
            'BALAMT' => (float) $this->ofx->BANKMSGSRSV1->STMTTRNRS->STMTRS->LEDGERBAL->BALAMT
        );

        unset($this->originalString);
        unset($this->ofx);
    }

    public function OFXParser($OFXContent = false) {
        if ($OFXContent)
            $this->loadFromString($OFXContent);
    }

    /*     * ************************************************************* */
    /*     * **             external functions to be used              *** */
    /*     * **            (advised to be used for filters)            *** */
    /*     * ************************************************************* */

    public function filter($what, $forWhat, $case = true, $like = false) {
        $case = !$case;
        $ret = Array();
        if ($case) {
            $what = strtoupper($what);
            $forWhat = strtoupper($forWhat);
        }
        for ($i = 0, $j = sizeof($this->moviments); $i < $j; $i++) {
            if ($like) {
                if (strstr($this->moviments[$i][$what], $forWhat))
                    $ret[] = $this->moviments[$i];
            }else {
                if ($this->moviments[$i][$what] == $forWhat)
                    $ret[] = $this->moviments[$i];
            }
        }
        return $ret;
    }

    public function getMoviments() {
        return $this->moviments;
    }

    public function getMemoLike($memo) {
        return $this->filter('MEMO', $memo, true, false);
    }

    public function getById($id) {
        return $this->filter('FITID', $id);
    }

    public function getByCheckNum($id) {
        return $this->filter('CHECKNUM', $id);
    }

    public function getCredits($min = 0) {
        $ret = Array();
        for ($i = 0, $j = sizeof($this->moviments); $i < $j; $i++) {
            if ($this->moviments[$i]['TRNTYPE'] == 'CREDIT' && $this->moviments[$i]['TRNAMT'] >= $min)
                $ret[] = $this->moviments[$i];
        }
        return $ret;
    }

    public function getByDate($month, $day, $year) {
        if ($month < 10)
            $month = '0' . $month;
        if ($day < 10)
            $day = '0' . $day;
        if ($year < 100)
            if ($year < 10)
                $year = '00' . $year;
            else
                $year = '0' . $year;
        $date = '/^' . $year . $month . $day . '([0-9])+$/';

        $ret = Array();
        for ($i = 0, $j = sizeof($this->moviments); $i < $j; $i++) {
            if (preg_match($date, $this->moviments[$i]['DTPOSTED'])) {
                $ret[] = $this->moviments[$i];
            }
        }
        return $ret;
    }

    public function getMoviment($mov) {
        if (isset($this->moviments[$mov]))
            return $this->moviments[$mov];
        else
            return false;
    }

    public function getDebits($min = 0) {
        $ret = Array();

        if ($min > 0)
            $min*= (-1);

        for ($i = 0, $j = sizeof($this->moviments); $i < $j; $i++) {
            if ($this->moviments[$i]['TRNTYPE'] == 'DEBIT' && $this->moviments[$i]['TRNAMT'] <= $min)
                $ret[] = $this->moviments[$i];
        }
        return $ret;
    }

    public function getBankingData(){
        return ($this->bankingData);
    }

    public function getFiledata(){
        return($this->fileData);
    }

}

?>